// 
// Amp Hott pour télémesure Graupner Hott
// 
// Version 0.14 
// Janvier 2021
//
// Auteur : www.wp.thyzoon.fr 
// 2020
// 
// Mesure du courant et tension batterie
//

//  Thisbip_alarme program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


// Evolutions
//
//  V0.13 --> V.14
//    Menu : Mode RESET Capacité : erreur de valeur mini
//    Calcul de la puissance maxi.
//    Type de variables (courant, puissance max) de 'int' à 'long'
//    Indication de version dans le menu
//    Resolution affichage du courant


//****************************************************//
//--                   Hardware                     --//
//      Equivalent : Arduino Pro Mini 5V 16Mhz        //
//               CPU : ATMega 328P                    //
//          Capteur de courant : ACS770               //
//****************************************************//    
//--                 PIN Mapping                    --//
//                                                    //
// D0 : RXD Réception Hott
// D1 : TXD Emission Hott
// D2 : Bouton poussoir de RESET capacite
// A0 : Courant (tension en sortie du capteur ACS770) 
// A1 : Tension batterie (diviseur R9 R10)
//****************************************************// 

#include "Arduino.h"
#include <EEPROM.h>
#include "Message.h"
#include "memo.h"
#include "Mesure.h"

// ---------------
// Fonctions init interruptions
void init_timer2 (); // Initialisation des interruptions du timer 2 pour les acquisitions analogiques

// Def LED verte carte Arduino mini pro (D13)
#define LEDPIN_PINMODE        pinMode (13, OUTPUT);
#define LEDPIN_OFF            PORTB &= ~(1<<5);
#define LEDPIN_ON             PORTB |= (1<<5);

#define BPCAP                 2   // Port PD2 -- INT0   pour le bouton poussoir
#define TP_CAPA_EEPROM        20000    // Temps en ms entre ecriture en EEPROM de la capacite

// Son des alarmes 
#define ALARME_CAPACITE         0x15  // code son Hott
#define ALARME_TENSION          0x12  // code son Hott
#define DUREE_MESSAGE_VOCAL 	  5000  // durée d'un message vocal en ms

// Baudrate UART pour Graupner Hott
#define SERIAL_COM_SPEED 19200

// ++++++++++++++++++++++++++++++++
// Variables parametres EEPROM
//
extern word mode_rst_capa;       // RESET capacité 0 = à chaque mise sous tension / 1 = par bouton poussoir / 2 = par seuil tension
extern word tension_mini;        // Tension mini par element
extern word type_bat;            // Nb element batterie
extern word capacite_max;        // capacité batterie maxi utilisable
extern word alarme_repet;        // temps de repetition alarme
extern word alarme_on_tension;   // alarme ON tension 
extern word alarme_on_capacite;  // alarme ON capacite
extern word capacite_conso;      // capacité batterie consommée
extern word type_acs;            // Id du type de capteur de courant ACS770
extern word offset_capteur_acs; // offset du capteur de courant

unsigned long temps_ecriture_capa;  // Temps (en ms) pour la prochaine écriture de la capacité consommee de EEPROM 
unsigned long temps_courant;        // Temps courant (en ms)

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Gestion Alarmes sonores et vocales Hott
//
byte bip_alarme;                     // son de l'alarme Hott
byte derniere_alarme;                // derniere alarme envoyée (son de l'alarme Hott)
boolean flag_alarm_capacite;         // Alarme capacite batterie
boolean flag_alarm_tension;          // Alarme tension batterie
word temps_alarme;                   // temps prochaine alarme
unsigned long Temps_derniere_alarme; // temp en ms

//*******************************************************
// Flag IT timer 2 / Acquisitions analogiques
boolean flag_acquisition;        // acquisitions analogiques valides

//----------------------------------
// valeur des acquisitions analogiques 'brutes' sur 10 bits
//
uint16_t mesure_courant;    // valeur acquistion du courant
uint16_t mesure_tension;    // valeur acquistion de la tension

//----------------------------------
// valeur des acquisitions analogiques traitees
float tension_bat;    // Tension batterie en V
long  courant_bat;     // Courant instantané consomme batterie en mA
long  courant_bat_max; // Courant instantané maxi batterie en mA
long  puissance_max;   // Puissance consommee maxi en mW
word mode_cal_offset_acs; // Pour la calibration de l'offset

//----------------------------------
// Gestion capacité de la batterie
float delta_capacite;     // Capacité consommée en 200ms en micro-ampères
float capacite_ua;        // Capacité totale consommée en micro-ampères
word capacite_ma;         // Capacité totale consommée en milli-ampères

// --------------------------------
// Gestion du bouton poussoir de RESET
byte bp_rst_capa;         // Détection et filtrage du BP  

// Gestion memorisation en EEPROM
Mem nvmem;

// Gestion messagerie Hott
GMessage message;

// Gestion des mesure de tension
Mesure analogique;

//********************************************
// Initialisations 
//
void setup() {

  // LED carte
  LEDPIN_PINMODE
  
  // Init communications 
  message.Init();

  // Init EEPROM  / Lecture parametres memorises
  nvmem.Init();
  
  // Init entrees analogiques
  analogique.Init();

  // Init entree bouton poussoir
  pinMode(BPCAP, INPUT);   // PD2 - INT0
  bp_rst_capa = 0;        // pas de détection 
  
  // Gestion alarmes
  flag_alarm_capacite = false; 
  flag_alarm_tension = false;               
     
  bip_alarme = 0;       // pas de son ou message vocal
  Temps_derniere_alarme = 0;
  derniere_alarme = 0;   // pas d'alarme 
   
  // Init interruptions pour acquisitions analogiques   
  flag_acquisition = false;
  init_timer2 ();      // Initialse les ITs timer 2 pour acquisitions analogiques

  // Gestion calibration Offset ACS
  mode_cal_offset_acs = 0;
  
  LEDPIN_ON 
  delay (250); // 250ms afin d'avoir la 1iere interruption Timer 2 pour acquisition de la tension
  
  // conversions des mesures
  courant_bat = (long) analogique.Courant_batterie (mesure_courant);
  tension_bat = analogique.Tension_batterie (mesure_tension);  
  courant_bat_max = courant_bat;
  puissance_max = courant_bat_max * tension_bat;
  
  if (mode_rst_capa == 2) {   // Mode RESET BP 
    // -----------------------------------------------
    // boucle de détection d'appui sur bouton poussoir RESET capacité si nécessaire
    for (int i = 0; i <= 999; i++) {  //environ 5 secondes  (1000x5ms)
      if (digitalRead(BPCAP)==0){ // Appui sur BP / pullup / Appui = 0
        bp_rst_capa++;     // comptage duree appui BP
        LEDPIN_OFF         // LED eteinte pendant appui BP
      }
      else 
        LEDPIN_ON
      delay(5);    // Tempo pour filtrage appui BP
    }
  LEDPIN_ON
  }
  
  // RESET capacité selon le mode choisi dans les menus
  capacite_ua = 0.0;   // par défaut RESET capacite
  capacite_ma = 0;
  
  switch (mode_rst_capa) {  // Selon mode RESET choisi dans le menu et en EEPROM
        case 1 : // Mode RESET capacité à chaque mise sous tension 
          //capacite_ua = 0.0;  // déjà fait si dessus
          break;
          
        case 2 : // Mode RESET capacité sur  bouton poussoir
          if (bp_rst_capa < 6) { //  Appui mini de 30ms (6 x 5ms)
           // Pas de RESET capacité donc lecture ancienne valeur en EEPROM
            capacite_ma = capacite_conso;                    // Lecture valeur memorise
            capacite_ua = (float) capacite_ma * 1000.0;      // conversion en micro-ampère 
          }
          break;
          
        case 3 : // Mode RESET capacité sur seuil de tension 4.15V par élément
          if (tension_bat < ((float) type_bat * 4.15))  {  // Seuil mini = 4.15V par elements
           // Pas de RESET capacité donc lecture ancienne valeur en EEPROM
            capacite_ma = capacite_conso;                // Lecture valeur memorise
            capacite_ua = (float) capacite_ma *1000.0;   // conversion en micro-ampère 
          } 
          else {  // RESET capacite 
            for (int i = 0; i <= 9; i++) {  // Clignotements rapide
              delay (200);
              LEDPIN_OFF
              delay (200);
              LEDPIN_ON 
            }
          }  
          break;
          
        default :
          break;
  }     
  LEDPIN_OFF
  flag_acquisition = false;  // RESET du flag des IT Timer 2

  temps_courant = millis ();                    // Temps courant (en ms)
  temps_ecriture_capa = temps_courant + TP_CAPA_EEPROM;  // Temps (en ms) pour la prochaine écriture de la capacité de EEPROM = temps courant + xxs
}

//******************************************
// Boucle principale
//
void loop() {   

   // ---------------------------------------------------------------
   // Traitement des acquisitions analogiques realisees en IT Timer 2
   // Periode = 200ms
   // 
   if (flag_acquisition) {   // 1 IT a eu lieu !!! 
     flag_acquisition = false;   // acquitement
      
     // Tension batterie en V
 	   tension_bat = analogique.Tension_batterie (mesure_tension);
	   // Courant consomme batterie en mA
	   courant_bat = (long) analogique.Courant_batterie (mesure_courant);
	  
	   // Calcul de la capacité consommé échantillonnée à 200ms, valeur en uAh pour limiter la somme des erreurs
	   delta_capacite = (float) courant_bat * 200.0 / 3600.0;
	  
	   // Capacité totale consommée en uAh
	   capacite_ua +=  delta_capacite;         // Total capacite consommee en uAh
     capacite_ma = (word) (capacite_ua / 1000.0);      // Conversion en mAh   

     // Memorise le courant maxi et puissance maxi (considere que la puissance maxi est atteinte au courant maxi)
     if (courant_bat > courant_bat_max) {
       courant_bat_max = courant_bat;
	     puissance_max = (long) ( ((float) courant_bat_max) * tension_bat);
     }  

     //-----------------------------------  
     // Alarme tension batterie
     // detection tension sous le seuil configuré
     if (alarme_on_tension)
       if (tension_bat < ((float) tension_mini / 100.0 * (float)type_bat))
         flag_alarm_tension = true;
       else
         flag_alarm_tension = false;  

     //-----------------------------------  
     // Alarme capacité batterie
     // detection capacité sous le seuil configuré
     if (alarme_on_capacite)
       if (capacite_ma > capacite_max)
         flag_alarm_capacite = true;
       else
        flag_alarm_capacite = false;  
   }

   //----------------------------------------------------------------------------------------------
   // Sauvegarde périodique en EEPROM de la capacité consommée
   // Important !! :
   //    Le nombre d'écriture en EEPROM est limité, car elle se dégrade
   //    Pour cela, la valeur n'est écrite que toutes les TP_CAPA_EEPROM ms et lorsque le courant est faible car 
   //      on considère que l'on ne coupe jamais l'alim lorsque la puissance consomme (moteur tournant)
   //

   temps_courant = millis();           // Temps courant (en ms)

   if ((mode_rst_capa)    // Si mode_rst_capa = 1 ou 2 , besoin de stocker la valeur, si 0 pas besoin
     && (courant_bat < 1000)     // courant < 1A  pas de puissance consommee.  
       && (temps_courant >= temps_ecriture_capa)) {            //  Temps pour ecriture en EEPROM atteint ou depasse
     nvmem.write_eprom(adr_eprom_capacite_conso,capacite_ma);   // Memo en EEPROM
     temps_ecriture_capa = temps_courant + TP_CAPA_EEPROM;       // Calcul du prochain temps d'écriture en EEPROM (+TP_CAPA_EEPROM ms)
   } 
  
  //----------------------------------------
  // Gestion de priorité des alarmes
  // Les radios MX-xx ne traitent qu'une alarme vocale à la fois,
  // Si l'on envoie plusieurs alarmes à la suite sans attendre la
  // fin du message vocal en cours, les alarmes suivantes sont perdues.
  //
  // Gestion du temps entre alarme afin de laisser assez de temps au message 
  // vocal d'être émis complètement avant envoi de l'alarme suivante
  
  temps_courant = millis();  // temps courant ms
  if ((bip_alarme == 0) && (temps_courant > (Temps_derniere_alarme + DUREE_MESSAGE_VOCAL))) {
    Temps_derniere_alarme = temps_courant;

    //--------------------------------
    // prise en compte des flags alarme 
    // Ordre inverse de la priorité
    //
    if ((flag_alarm_tension)&&(derniere_alarme!=ALARME_TENSION)) {   
	    flag_alarm_tension = false;              // prise en compte
	    bip_alarme = ALARME_TENSION;             // son alarme tension  
    }

    if ((flag_alarm_capacite)&&(derniere_alarme!=ALARME_CAPACITE)){   
	    flag_alarm_capacite = false;              // prise en compte
	    bip_alarme = ALARME_CAPACITE;             // son alarme capacite
    }   
    
    if (bip_alarme)
      derniere_alarme = bip_alarme;// memorise la dernière alarme.
    else
      derniere_alarme = 0;
  }

  //-----------------------------------------------
  // Gestion des messages Hott / Menus et telemesure
  // 
  if (message.main()) {
    bip_alarme = 0;   // remise à zero de l'alarme après une émission

    // Traitement d'une demande de calibration Offset capteur ACS (Menu radio)
    if (mode_cal_offset_acs == 1) {    // demande calibration Offset en cours
      offset_capteur_acs = mesure_courant;      // Offset = derniere mesure !! pas de courant dans le capteur !!
      nvmem.write_eprom(adr_eprom_offset_acs,offset_capteur_acs);    // memorise en EEPROM le nouvel OFFSET
      mode_cal_offset_acs = 2;   // Calibration offset realisee    
    }
    
  }    ///---- fin 'if (message.main())'
}
