// 
// pour télémesure Graupner Hott
// 
// Gestion messagerie Hott
//
// Auteur : www.wp.thyzoon.fr 
// 2016-2020
//

#include "Message.h"
#include "Mesure.h"
#include "memo.h"
//#include "acsr.h"
#include <EEPROM.h>

#define VERSION         "0.14"

// buffer message
static uint8_t _hott_serial_buffer[173];   //création d'une variable tampon pour stocker les struct

// pointer des structures vers le tampon "_hott_serial_buffer"
struct HOTT_GAM_MSG       *hott_gam_msg = (struct HOTT_GAM_MSG *)&_hott_serial_buffer[0];
struct HOTT_TEXTMODE_MSG	*hott_txt_msg =	(struct HOTT_TEXTMODE_MSG *)&_hott_serial_buffer[0];

// Variable valeurs EEPROM
extern word mode_rst_capa;      // 0 = RESET capacité à chaque mise sous tension  1 = RESET par bouton poussoir 2 = Seuil de tension
extern word tension_mini;       // Tension mini par element
extern word typ_bat;            // Nb element batterie
extern word capacite_max;       // capacité batterie maxi utilisable
extern word alarme_repet;       // temps de repetition alarme
extern word alarme_on_tension;  // alarme ON tension 
extern word alarme_on_capacite; // alarme ON capacite
extern word capacite_conso;     // capacité batterie consommée

extern byte bip_alarme;         // son de l'alarme

// For communication
static uint8_t octet1 = 0;  // reception
static uint8_t octet2 = 0;  // reception

// Gestion des mesures 
extern Mesure analogique;

extern float tension_bat;    // Tension batterie en V
extern long  courant_bat;    // Courant instantané consomme batterie en mA
extern word  capacite_ma;    // Capacité totale consommée en milli-ampères

extern long  courant_bat_max; // Courant instantané maxi batterie en mA
extern long  puissance_max;   // Puissance maxi en mW

extern word mode_cal_offset_acs; // Pour la calibration de l'offset
word aff_cal_offset_acs;         // Pour menu calibration de l'offset

int ent,dec;     // pour conversion et afficchage float --> texte

//---------------------------------
// Capteur de courant
extern word  type_acs;       // Id du type de capteur de courant
extern char *ACSref[];       // Texte references des capteurs ACS770 pour affichages menus 

//---------------------------------
// Batterie
extern word type_bat;        // 2S ... 6S

// Gestion memorisation en EEPROM
extern Mem nvmem;

GMessage::GMessage(){

}
void GMessage::Init(){
// Initialisation COM 19200 bauds pour recepteur HOTT GRxx
  Serial.begin (SERIAL_COM_SPEED); 
  aff_cal_offset_acs = 0;
}

// Initialisation message GAM (General Air Module)
void GMessage::init_gam_msg(){
  //met tous à Zero, puis on modifie les constantes
  memset(hott_gam_msg, 0, sizeof(struct HOTT_GAM_MSG));   
  hott_gam_msg->start_byte = 0x7c;
  hott_gam_msg->gam_sensor_id = HOTT_TELEMETRY_GAM_SENSOR_ID;
  hott_gam_msg->sensor_id = 0xd0;
  hott_gam_msg->stop_byte = 0x7d;
}

// Emission de la trame
void GMessage::send(int lenght){ 
  uint8_t sum = 0;
  delay(5);
  for(int i = 0; i < lenght-1; i++){
    sum = sum + _hott_serial_buffer[i];
    Serial.write (_hott_serial_buffer[i]);
    delayMicroseconds(HOTTV4_TX_DELAY);

    Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
  }  
  //Emision du checksum
  Serial.write (sum);
  delayMicroseconds(HOTTV4_TX_DELAY);
  Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
}

int GMessage::main(){ 
  // 
  static byte page_settings = 1;//numéro de la page de settings à afficher 
    
  if (Serial.available() >= 2) {
    uint8_t octet1 = Serial.read();
    switch (octet1) {
    case HOTT_BINARY_MODE_REQUEST_ID:
      { 
        uint8_t  octet2 = Serial.read();        
        // Demande RX Module =	$80 $XX
        switch (octet2) {
        case HOTT_TELEMETRY_GAM_SENSOR_ID: //0x8D
        {
            // init structure
            init_gam_msg();        
           
            // Set current instantanne (A) en 0.1 A
            setAmpere((uint16_t) (courant_bat/100));    

            // Set capacite mA/h
            setMilliAmpere((uint16_t) (capacite_ma));  

            // Tensions 
			      setMainVoltage (tension_bat);
            setVoltage1 (0);
            setVoltage2 (0);

            // Temperatures 1 et 2
            setTemp(0,1);
            setTemp(0,2);
        
            // Gestion Alarme : envoie du code son ou message vocal          
            alarme (bip_alarme); 
              
            // altitude 0 et montée 0
            setAltitudeInMeters (0);
            setClimbrate (0);
            setClimbrate_M3 (0);              
            
            setRPM (0); // ecriture RPM
            
            send(sizeof(struct HOTT_GAM_MSG));
            break;
          } //end case GAM*/
        } //end case octet 2
        break;
      }

    case HOTT_TEXT_MODE_REQUEST_ID:
      {      
        uint8_t  octet3 = Serial.read();
        byte id_sensor = (octet3 >> 4);
        byte id_key = octet3 & 0x0f;
        static byte ligne_select = 2 ; // Ligne 2 selectionnée par défaut
        static int8_t ligne_edit = -1 ;
        hott_txt_msg->start_byte = 0x7b;
        hott_txt_msg->esc = 0;
        hott_txt_msg->warning_beeps = 0;
        
        memset((char *)&hott_txt_msg->text, 0x20, HOTT_TEXTMODE_MSG_TEXT_LEN);
        hott_txt_msg->stop_byte = 0x7d;
        
          if (id_sensor == (HOTT_TELEMETRY_GAM_SENSOR_ID & 0x0f)) 
          {            
            switch (page_settings) { // Pages de parametres 
              
              case 1:   //PAGE 1 -------------------------- 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(5,ligne_select + 1); // Ligne 5 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(1,ligne_select - 1); //Ligne 1 mini
                          break; 
                          
                        case HOTT_KEY_SET :                      // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :        // Sortie
                          hott_txt_msg->esc = 0x01;
                          ligne_edit = -1 ;
                          ligne_select = 1;
                          break;  
                                           
                        case HOTT_KEY_RIGHT :      // passage en page 2              
                          page_settings = 2; 
                          ligne_edit = -1 ;     
                          ligne_select = 2;                     
                          break;    
                        
                        default : break;
                      }
                    }   
                      
                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Mode de RESET de la capacité à l'initialisation
                    else if (ligne_select == 1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          mode_rst_capa ++;  
                          if (mode_rst_capa > 3)
                             mode_rst_capa =1;                  
                        break;
                          
                        case HOTT_KEY_DOWN :
                          mode_rst_capa --;
                          if (mode_rst_capa < 1)
                             mode_rst_capa =3;                 
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_mode,mode_rst_capa);  // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Type de capteur de courant
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                           if (type_acs < 8)     // 8 max
                             type_acs ++;                   
                          break;
                          
                        case HOTT_KEY_DOWN :
                           if (type_acs > 0)
                             type_acs--;  
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_type_acs,type_acs); 
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Tension mini element 
                    else if (ligne_select == 3)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          tension_mini = tension_mini + 5;     // volt x 100
                          if (tension_mini > 440) // 4.40V
                            tension_mini = 440;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          tension_mini = tension_mini - 5;      // volt x 100
                          if (tension_mini < 250)  // 2.50V
                              tension_mini = 250;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_v_mini,tension_mini); 
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }
                        
                    //*************************************************
                    //LIGNE 5 SELECTIONNEE = Nb elements de la batterie
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (type_bat < 6)   // 6S
                            type_bat++;                     
                          break;
                          
                        case HOTT_KEY_DOWN :    
                          if (type_bat > 2)   // 2S
                              type_bat --;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_type_bat,type_bat); 
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 

                    //*************************************************
                    //LIGNE 6 SELECTIONNEE = Capacité maxi 
                    else if (ligne_select == 5)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          capacite_max = capacite_max + 100;     
                          if (capacite_max > 10000) 
                            capacite_max = 10000;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          capacite_max = capacite_max - 100;      
                          if (capacite_max < 1000) 
                              capacite_max = 1000;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_capacite_max,capacite_max); 
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }                     
                             
                    //******************************************************************************************
                    // Affichage de la page 1 du menu
                    //
                    // ligne 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * AmpHott  V%s * ",VERSION);
					       
                    // ligne 1: Mode de RESET de la capacité à l'initialisation
                    // 1 = RESET capacité à chaque mise sous tension  2 = RESET par bouton poussoir  3 = Seuil 4.15V / elements
                    if (mode_rst_capa == 1)
                      snprintf((char *)&hott_txt_msg->text[1],21," Mode : Power ON    ");
                    else if  (mode_rst_capa == 2)
                      snprintf((char *)&hott_txt_msg->text[1],21," Mode : Bouton Pous.");
                    else if  (mode_rst_capa == 3)
                      snprintf((char *)&hott_txt_msg->text[1],21," Mode : Seuil 4.15V ");

                    // ligne 2 : Type du capteur ACS770
                    snprintf((char *)&hott_txt_msg->text[2],21," Capteur %s",ACSref[type_acs]);
                    
                    // ligne 3: Tension mini element
                    snprintf((char *)&hott_txt_msg->text[3],21," V Elem mini   %i.%iV",int(tension_mini/100),int(tension_mini%100));
                        
                    //ligne 4: Nb elements batterie    type_bat
                    snprintf((char *)&hott_txt_msg->text[4],21," Type batterie : %iS",type_bat);

                    //ligne 5:
                    snprintf((char *)&hott_txt_msg->text[5],21," Capacite %5i mAh",capacite_max); 
                    
                    //ligne 6:
                    snprintf((char *)&hott_txt_msg->text[6],21,"                    "); 
                    	           
                    //ligne 7:
                    snprintf((char *)&hott_txt_msg->text[7],21," thyzoon.fr     2021"); 
                    
                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;                    
              }//END PAGE 1
                  
              //-------------------------------------------------------------------------------------------------    
              case 2:             //PAGE 2 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(3,ligne_select + 1); // // Ligne 3 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(0,ligne_select - 1); // // Ligne 0 mini
                          break; 
                          
                        case HOTT_KEY_SET :   // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :    // passage en page 1  
                          page_settings = 1;
                          ligne_edit = -1;
                          ligne_select = 2;                           
                          break;
                          
                         case HOTT_KEY_RIGHT :  // derniere page                 
                          break;  
                                                   
                        default : break;
                      }
                    }   

                   //*************************************************
                    //LIGNE 1 SELECTIONNEE = Alarme Tension ON / OFF
                    else if (ligne_select == 0)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_on_tension == 0)
                             alarme_on_tension = 1;                  
                          else
                             alarme_on_tension = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_on_tension == 0)
                             alarme_on_tension = 1;                  
                          else
                             alarme_on_tension = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_alarme_tension,alarme_on_tension); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }

                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Alarme Capacité ON / OFF
                    else if (ligne_select == 1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_on_capacite == 0)
                             alarme_on_capacite = 1;                  
                          else
                             alarme_on_capacite = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_on_capacite == 0)
                             alarme_on_capacite = 1;                  
                          else
                             alarme_on_capacite = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_alarme_capacite,alarme_on_capacite);  // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }                   
                        
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Delai de repetition de l'alarme
                    else if (ligne_select == 2)
                    {
                       switch (id_key) {
                        case HOTT_KEY_UP :
                          alarme_repet = alarme_repet + 15;   // secondes
                          if (alarme_repet > 60)   // secondes
                            alarme_repet = 60;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          alarme_repet = alarme_repet - 15;  // secondes
                          if (alarme_repet <= 15)  
                              alarme_repet = 15;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          nvmem.write_eprom(adr_eprom_alarme_repet,alarme_repet);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }                     
                    } 

                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Calibration de l'offset du capteur ACS
                    else if (ligne_select == 3)
                    {
                       switch (id_key) {
                        case HOTT_KEY_UP :
                          if (aff_cal_offset_acs ==0)   // secondes
                            aff_cal_offset_acs = 1;                     
                          else 
                            aff_cal_offset_acs = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (aff_cal_offset_acs ==0)   // secondes
                            aff_cal_offset_acs = 1;                     
                          else 
                            aff_cal_offset_acs = 0;  
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          if (aff_cal_offset_acs ==1)  // demande calibration dans le menu
                            mode_cal_offset_acs = 1;                            
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }                     
                    } 
                                                   
                    //******************************************************************************************
                    // Affichage de la page 2 du menu
					      
                      //ligne 0:
                      if (alarme_on_tension == 1)
                        snprintf((char *)&hott_txt_msg->text[0],21," Alarme Tension  ON ");
                      else 
                        snprintf((char *)&hott_txt_msg->text[0],21," Alarme Tension  OFF");
                        
                      //ligne 1:
                      if (alarme_on_capacite == 1)
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarme Capacite ON ");
                      else 
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarme Capacite OFF");
                        
                      //ligne 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Repetition chro %2is",alarme_repet);

                      //ligne 3:
                      if (aff_cal_offset_acs ==0)
                          snprintf((char *)&hott_txt_msg->text[3],21," Cal Offset ACS --- ");
                      else if  (aff_cal_offset_acs ==1) // Demande
                          snprintf((char *)&hott_txt_msg->text[3],21," Cal Offset ACS GO  ");
                      if (mode_cal_offset_acs == 2 )   // Termine
                          snprintf((char *)&hott_txt_msg->text[3],21," Cal Offset ACS OK  ");                          
                                                                  
                      //ligne 4:
                      ent = abs(courant_bat/1000);
                      dec = abs((courant_bat%1000)/100);
                      snprintf((char *)&hott_txt_msg->text[4],21," Courant = %3d.%1dA",ent,dec);     
                      
                      //ligne 5:
                      ent = abs(puissance_max/1000);
                      snprintf((char *)&hott_txt_msg->text[5],21," Pmax = %4iW",ent);       
                      
                      //ligne 6:
                      ent = abs(courant_bat_max/1000);
                      dec = abs((courant_bat_max%1000)/100);                     
                      snprintf((char *)&hott_txt_msg->text[6],21," Imax = %3i.%1iA",ent,dec);   
                      
                      //ligne 7:
                      snprintf((char *)&hott_txt_msg->text[7],21,"                    ");
                                          
                      hott_txt_msg->text[ligne_select][0] = '>';
                      _hott_invert_ligne(ligne_edit);
                    break;                    
                  }//END PAGE 2   
                  
                  default: //PAGE 
                  {
                    break;
                  }            
                 
            }//END SETTINGS 
            // Emission en mode texte
			_hott_send_text_msg();  // Trame GAM
          } // END IF :  id_sensor = HOTT_TELEMETRY_GAM_SENSOR_ID 
                  
        break;
      }
    }	
    return (1);  // Emission
  }
  else
   return (0);   // Pas d'emission
}

//////////////////////////////////////////////////
// Tension principale (resolution 0.1V)
void GMessage::setMainVoltage(float tension){
  hott_gam_msg->main_voltage = (uint16_t) (tension * 10);
}

// Batterie 1 (resolution 0.1V)
void GMessage::setVoltage1(float volt){
  hott_gam_msg->Battery1 =  (uint16_t) (volt * 10); 
}

// Batterie 2 (resolution 0.1V)
void GMessage::setVoltage2(float volt){
  hott_gam_msg->Battery2 = (uint16_t) (volt * 10); 
}

// Element Lipo n (resolution 0.02V)
void GMessage::setLipo(float volt, int lipo){

  if (lipo >= 1 and lipo <= 6)
  {
    lipo--;
    hott_gam_msg->cell[lipo] =  (uint16_t) 100 * volt / 2 ; 
    
    if (volt/2 <= hott_gam_msg->min_cell_volt || hott_gam_msg->min_cell_volt ==0)
    {
      hott_gam_msg->min_cell_volt = (uint16_t) 100 * volt/2 ;
      hott_gam_msg->min_cell_volt_num = lipo;
    }
  }
}

// Altitude relative en metres : -500 / +9999 [m]
void GMessage::setAltitudeInMeters(uint16_t alti){
  hott_gam_msg->altitude = alti + 500;
}

// Temperature : -20 a +235° C 
// 
void GMessage::setTemp(int temp, int capteur){
  if(temp < -20)
    temp = -20;
  else if(temp > 234)
    temp = 235;
  if (capteur < 1)
    capteur = 1;
  if (capteur > 2)
    capteur = 2; 
  if (capteur == 1)
    hott_gam_msg->temperature1 = temp + 20;
  else if (capteur == 2)
    hott_gam_msg->temperature2 = temp + 20;
}

// Niveau reservoir : 0%, 25%, 50%, 75%, 100%  
// 
void GMessage::setFuelPercent(uint8_t pourcent){
  if(pourcent > 100)
    pourcent = 100;
  else if(pourcent < 0)
    pourcent = 0;
  hott_gam_msg->fuel_procent = pourcent ;
}

// niveau reservoir en ml : 0 a 65535 ml
// 
void GMessage::setFuelMilliliters(uint16_t ml){
  hott_gam_msg->fuel_ml = ml ;
}

// Rotation : 0 a 655350 tr/min
// 

void GMessage::setRPM(uint16_t rpm){
  hott_gam_msg->rpm = rpm ;
} 

// Set climbrates
void GMessage::setClimbrate(uint16_t climb_L){
  hott_gam_msg->climbrate_L =  climb_L+30000;
}

void GMessage::setClimbrate_M3(int climb3s){
  hott_gam_msg->climbrate3s =  climb3s+120;
}

// Set speed
void GMessage::setSpeed(float speed){
  hott_gam_msg->climbrate3s =  (int) speed;
}


// Set current  (0.1A)    !!!!  0.1A
void GMessage::setAmpere(uint16_t amp){
	hott_gam_msg->current =  (uint16_t) (amp); 
}

// Set capacite mA/h
void GMessage::setMilliAmpere(uint16_t mA){
	hott_gam_msg->batt_cap =  (uint16_t) (mA / 10); 
}

//
// Alarme sonore sur radio
//
void GMessage::alarme (uint8_t son){
  hott_gam_msg->warning_beeps = son;
}

void GMessage::_hott_send_text_msg() {
  for(byte *_hott_msg_ptr = hott_txt_msg->text[0]; _hott_msg_ptr < &hott_txt_msg->stop_byte ; _hott_msg_ptr++){
    if (*_hott_msg_ptr == 0)
      *_hott_msg_ptr = 0x20;
  }  
  //_hott_send_msg(_hott_serial_buffer, sizeof(struct HOTT_TEXTMODE_MSG));
  send(sizeof(struct HOTT_TEXTMODE_MSG));
}

char * GMessage::_hott_invert_all_chars(char *str) {
  return _hott_invert_chars(str, 0);
}

char * GMessage::_hott_invert_chars(char *str, int cnt) {
  if(str == 0) return str;
  int len = strlen(str);
  if((len < cnt)  && cnt > 0) len = cnt;
  for(int i=0; i< len; i++) {
    str[i] = (byte)(0x80 + (byte)str[i]);
  }
  return str;
}

void GMessage::_hott_invert_ligne(int ligne) {
  if (ligne>= 0 && ligne<= 7)
    for(int i=0; i< 21; i++) {
      if (hott_txt_msg->text[ligne][i] == 0)   //inversion du caratère null (fin de string)
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + 0x20);
      else
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + (byte)hott_txt_msg->text[ligne][i]);
    }
}
