// 
// Hott pour télémesure Graupner Hott
// 
// Gestion memorisation en EEPROM
//
// Auteur : www.thyzoon.fr 
// 2016-2020
//

#include "Arduino.h"
#include <EEPROM.h>
#include "memo.h"

// Variables valeurs EEPROM
word mode_rst_capa;      // 1 = RESET capacité à chaque mise sous tension  2 = RESET par bouton poussoir   3 = seuil de tension
word tension_mini;       // Tension mini par element
word type_bat;           // Nb element batterie
word capacite_max;       // capacité batterie maxi utilisable
word alarme_repet;       // temps de repetition alarme
word alarme_on_tension;  // alarme ON tension 
word alarme_on_capacite; // alarme ON capacite
word capacite_conso ;    // capacité batterie consommée
word type_acs;           // type capteur de courant
word offset_capteur_acs; // offset du capteur de courant

//word sensibilite_capteur_acs; // sensibilite du capteur de courant x10   par ex : 205 = 20.5 mV/A

Mem::Mem(){

}

void Mem::Init () {
  // ******************************************************
  // Test for 1st time init of the Arduino (First power on)
  int test = read_eprom(adr_eprom_init);
  
  if (test != 012)
  {  // initialisation EEPROM pour la 1iere fois
    write_eprom(adr_eprom_init,012);
	  write_eprom(adr_eprom_mode,MODE_RESET);      // 1 = RESET capacité à chaque mise sous tension  2 = RESET par bouton poussoir 3 = seuil de tension
    write_eprom(adr_eprom_v_mini,TENSION_MIN_EL); 
    write_eprom(adr_eprom_type_bat,4);                     // Type 4S
    write_eprom(adr_eprom_capacite_max,CAPACITE_MAX); 
    write_eprom(adr_eprom_alarme_repet,ALARME_REPET); 
    write_eprom(adr_eprom_alarme_tension,ALARME_OFF); 
    write_eprom(adr_eprom_alarme_capacite,ALARME_OFF); 
    write_eprom(adr_eprom_capacite_conso,0);
    write_eprom(adr_eprom_type_acs,0);                    // type "non defini" par défaut voir table dans le fichier acsr.h
    write_eprom(adr_eprom_offset_acs,102);                // Offset nominal capteur ACS Unidirectionnel
  }
  
  // Lectures EEPROM
  mode_rst_capa = read_eprom(adr_eprom_mode);
  tension_mini = read_eprom(adr_eprom_v_mini);                      // Lecture de la tension mini par element
  type_bat = read_eprom(adr_eprom_type_bat);                        // Type de batterie
  capacite_max = read_eprom(adr_eprom_capacite_max);                // capacité batterie maxi utilisable
  alarme_repet  = read_eprom(adr_eprom_alarme_repet);               // temps de repetition
  alarme_on_tension = read_eprom(adr_eprom_alarme_tension);         // Activation alarme tension
  alarme_on_capacite = read_eprom(adr_eprom_alarme_capacite);       // Activation alarme capacite
  capacite_conso = read_eprom(adr_eprom_capacite_conso);            // Capacité consommee utilise selon le mode de RESET
  type_acs = read_eprom(adr_eprom_type_acs);                        // Type de capteur
  offset_capteur_acs = read_eprom(adr_eprom_offset_acs);            // Offset du capteur ACS
}

// Lecture d'un mot de 16 bits en EEPROM
uint16_t Mem::read_eprom(int address){
  return  (uint16_t) EEPROM.read(address) * 256 + EEPROM.read(address+1) ;
}

// Ecriture d'un mot de 16 bits en EEPROM
void Mem::write_eprom(int address,uint16_t val){
  EEPROM.write(address, val  / 256);
  EEPROM.write(address+1,val % 256 );
}
